/*
 * AbstractSnowlet.java
 *
 * Copyright (C) 2010 Alessio Stalla
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * As a special exception, the copyright holders of this library give you
 * permission to link this library with independent modules to produce an
 * executable, regardless of the license terms of these independent
 * modules, and to copy and distribute the resulting executable under
 * terms of your choice, provided that you also meet, for each linked
 * independent module, the terms and conditions of the license of that
 * module.  An independent module is a module which is not derived from
 * or based on this library.  If you modify this library, you may extend
 * this exception to your version of the library, but you are not
 * obligated to do so.  If you do not wish to do so, delete this
 * exception statement from your version.
 */

package snow;

import javax.script.Bindings;
import com.jgoodies.binding.PresentationModel;

public abstract class AbstractSnowlet implements Snowlet {

    private PresentationModel presentationModel;
    private Object backingBean;

    protected Bindings createBindings() {
	Bindings b = Snow.getScriptEngine().createBindings();
	b.put("snow:*backing-bean*", backingBean);
	b.put("snow:*presentation-model*", presentationModel);
	return b;
    }

    public Object eval() {
	try {
	    return eval(createBindings());
	} catch(Exception e) {
	    throw new RuntimeException("Exception while evaluating snowlet " + this, e);
	}
    }

    protected abstract Object eval(Bindings bindings) throws Exception;

    public PresentationModel getPresentationModel() {
	return presentationModel;
    }

    public void setPresentationModel(PresentationModel presentationModel) {
	this.presentationModel = presentationModel;
    }

    public Object getBackingBean() {
	return backingBean;
    }

    public void setBackingBean(Object backingBean) {
	this.backingBean = backingBean;
    }

}